==> django_spellbook-0.1.13b5.tar.gz: django_spellbook-0.1.13b5/tests/test_djangolike_integration.py <==
import unittest
import markdown
import io
import re
import os # Needed for path handling and directory creation
from django.test import TestCase # Or use unittest.TestCase
# Adjust the import path according to your project structure
from django_spellbook.markdown.extensions.django_like import DjangoLikeTagExtension
from django_spellbook.markdown.parser import MarkdownParser

from django_spellbook.management.commands.spellbook_md_p.reporter import MarkdownReporter

# Define relative paths
INPUT_MD_DIR = 'markdown_testers'
INPUT_MD_FILENAME = 'test_djangolike_blog.md'
OUTPUT_HTML_DIR = 'html_testers'
OUTPUT_HTML_FILENAME = 'test_djangolike_blog_parser_output.html'
GOLDEN_HTML_DIR = 'html_goldens' # New directory for golden file
GOLDEN_HTML_FILENAME = 'test_djangolike_blog.html'

class TestDjangoLikeIntegrationWithParser(TestCase):
    """
    Performs an end-to-end integration test using the application's
    MarkdownParser class. Includes focused assertions targeting known issues
    and comparison against an OPTIMAL golden HTML file.
    """

    @classmethod
    def setUpClass(cls):
        """
        Reads markdown input file, processes via MarkdownParser, stores HTML string,
        and writes HTML output file once for the class. Loads OPTIMAL golden file.
        """
        super().setUpClass()

        cls.base_dir = os.path.dirname(os.path.abspath(__file__))
        cls.input_md_path = os.path.join(cls.base_dir, INPUT_MD_DIR, INPUT_MD_FILENAME)

        if not os.path.exists(cls.input_md_path):
            raise FileNotFoundError(f"MD input file not found: {cls.input_md_path}")

        try:
            with open(cls.input_md_path, 'r', encoding='utf-8') as f:
                markdown_text = f.read()
        except Exception as e:
            raise IOError(f"Error reading MD file {cls.input_md_path}: {e}")

        cls.reporter_buffer = io.StringIO()
        cls.reporter = MarkdownReporter(cls.reporter_buffer)
        try:
            parser = MarkdownParser(markdown_text=markdown_text, reporter=cls.reporter)
            cls.html_output = parser.get_html()
        except Exception as e:
             raise RuntimeError(f"MarkdownParser failed during setUpClass: {e}")

        cls.output_html_dir_path = os.path.join(cls.base_dir, OUTPUT_HTML_DIR)
        cls.output_html_file_path = os.path.join(cls.output_html_dir_path, OUTPUT_HTML_FILENAME)
        try:
            os.makedirs(cls.output_html_dir_path, exist_ok=True)
            with open(cls.output_html_file_path, 'w', encoding='utf-8') as f:
                f.write(cls.html_output)
        except Exception as e:
            print(f"\n[Test Warning] Could not write actual HTML output file {cls.output_html_file_path}: {e}")

        cls.golden_html_path = os.path.join(cls.base_dir, GOLDEN_HTML_DIR, GOLDEN_HTML_FILENAME)
        if not os.path.exists(cls.golden_html_path):
            cls.expected_html = None
            print(f"\n[Test Warning] OPTIMAL golden file not found: {cls.golden_html_path}. Golden test will fail.")
        else:
            try:
                with open(cls.golden_html_path, 'r', encoding='utf-8') as f:
                    cls.expected_html = f.read()
            except Exception as e:
                cls.expected_html = None
                print(f"\n[Test Warning] Error reading OPTIMAL golden file {cls.golden_html_path}: {e}. Golden test will fail.")

    # --- Golden File Test (Ultimate Goal) ---
    @unittest.expectedFailure
    def test_against_optimal_golden_file(self):
        """Compares the generated HTML against the OPTIMAL golden file."""
        self.assertIsNotNone(self.expected_html, f"OPTIMAL Golden file could not be loaded: {self.golden_html_path}")
        # This test WILL FAIL until all processor bugs and MD rendering issues are fixed.
        self.assertMultiLineEqual(
            self.html_output.strip(),
            self.expected_html.strip(),
            "Generated HTML does not match OPTIMAL golden file. Fix processor bugs!"
            )

    # --- Assertions Targeting Specific Remaining Issues ---

    @unittest.expectedFailure
    def test_markdown_lists_inside_custom_tag(self):
        """
        BUG CHECK: Asserts Markdown lists render correctly inside custom tags.
        (Currently fails - list renders as literal text with <br /> inside <p>).
        """
        expected_list_html = '<ul>\n<li>As should lists.</li>\n<li>Item two.</li>\n</ul>'
        section_regex = r'<section\s+.*?class="hero".*?id="hero-section".*?>(.*?)<\/section>'
        match = re.search(section_regex, self.html_output, re.DOTALL)
        self.assertTrue(match, "Could not find the hero section tag.")
        section_content = match.group(1) if match else ""
        self.assertIn(expected_list_html, section_content, "Markdown list did not render correctly inside section tag.")
        # Also assert the incorrect rendering is NOT present
        self.assertNotIn("- As should lists.<br />", section_content, "List appears to be rendering literally inside section.")

    @unittest.expectedFailure
    def test_no_paragraph_splitting_around_inline_django_tags(self):
        """
        BUG CHECK: Asserts paragraphs are NOT split incorrectly around inline django tags.
        (Currently fails - paragraphs split and spurious `<code>`/`<p>` tags appear).
        """
        # Check for the known *incorrect* pattern (split paragraph + potential code tag)
        incorrect_pattern_static = "<p>`: </p>\n<django-tag>{% static 'css/blog_styles.css' %}</django-tag>" # Example check
        incorrect_pattern_url = "<p>`: </p>\n<django-tag>{% url 'post_detail' slug='test-drive-post' %}</django-tag>" # Example check

        self.assertFalse(incorrect_pattern_static in self.html_output, "Paragraph incorrectly split around static tag.")
        self.assertFalse(incorrect_pattern_url in self.html_output, "Paragraph incorrectly split around url tag.")

        # Check that the OPTIMAL structure (single paragraph) is present
        # This will fail until fixed.
        optimal_inline_block = "<p><code>Let's link to a static asset</code>: <django-tag>{% static 'css/blog_styles.css' %}</django-tag><br />\n<code>And here's a dynamic URL:</code> <django-tag>{% url 'post_detail' slug='test-drive-post' %}</django-tag><br />\nWe can even include another template snippet: <django-tag>{% include 'includes/sidebar.html' %}</django-tag></p>"
        self.assertIn(optimal_inline_block, self.html_output, "Inline django tags and text not rendering correctly in single paragraph.")

    @unittest.expectedFailure
    def test_django_else_tag_preservation(self):
        """
        BUG CHECK: Asserts '{% else %}' is preserved via <django-tag>, not converted to <else>.
        (Currently fails - renders as <else>...</else>).
        """
        # Assert that the incorrect <else> tag is NOT present
        self.assertNotIn('<else>', self.html_output, "Found unexpected <else> tag.")
        self.assertNotIn('</else>', self.html_output, "Found unexpected </else> tag.")
        # Assert that the correct <django-tag> wrapper IS present
        self.assertIn('<django-tag>{% else %}</django-tag>', self.html_output, "Expected <django-tag>{% else %}</django-tag> not found.")

    def test_custom_end_tags_are_consumed(self):
        """
        BUG CHECK: Asserts that custom end tags are REMOVED from the final output.
        (This *should* pass now for section/article based on the latest HTML).
        We removed the complex div case where it failed before. Add specific failing cases if needed.
        """
        # These should now pass for the current markdown content
        self.assertFalse('{% enddiv %}' in self.html_output, "Found literal '{% enddiv %}' in output.")
        self.assertFalse('{% endsection %}' in self.html_output, "Found literal '{% endsection %}' in output.")
        self.assertFalse('{% endaside %}' in self.html_output, "Found literal '{% endaside %}' in output.")
        self.assertFalse('{% endarticle %}' in self.html_output, "Found literal '{% endarticle %}' in output.")

    @unittest.expectedFailure
    def test_heading_inside_custom_tag(self):
        """
        ISSUE CHECK: Asserts H2 renders correctly inside the article tag.
        (Currently fails - renders as literal '##' inside <p>).
        """
        article_regex = r'<article\s+.*?id="post-body".*?>(.*?)<\/article>'
        match = re.search(article_regex, self.html_output, re.DOTALL)
        self.assertTrue(match, "Could not find the article tag.")
        article_content = match.group(1) if match else ""
        # This assertion will fail
        self.assertIn('<h2>Sub-heading Inside Article</h2>', article_content, "H2 did not render correctly inside article.")
        # Check for the incorrect rendering
        self.assertIn('<p>## Sub-heading Inside Article<br />', article_content, "H2 seems to be rendering literally inside <p> tag.")


    def test_aside_content_is_paragraph(self):
        """
        ISSUE CHECK: Asserts that indented content inside <aside> renders as <p>, not <pre><code>.
        (Currently fails due to source indentation + standard MD behavior).
        """
        aside_regex = r'<aside\s+.*?class="pull-quote".*?>(.*?)<\/aside>'
        match = re.search(aside_regex, self.html_output, re.DOTALL)
        self.assertTrue(match, "Could not find the aside tag.")
        aside_content = match.group(1) if match else ""
        # This assertion will fail
        self.assertIn('<p>This is an aside (pull quote)', aside_content, "Aside content should be a paragraph.")
        self.assertNotIn('<pre><code>', aside_content, "Aside content rendered unexpectedly as pre/code.")


# Allows running the tests directly from the command line
if __name__ == '__main__':
    unittest.main()

==> django_spellbook-0.1.15b4.tar.gz: django_spellbook-0.1.15b4/tests/test_djangolike_integration.py <==
import unittest
import markdown
import io
import re
import os # Needed for path handling and directory creation
from django.test import TestCase # Or use unittest.TestCase
# Adjust the import path according to your project structure
from django_spellbook.markdown.extensions.django_like import DjangoLikeTagExtension
from django_spellbook.markdown.parser import MarkdownParser

from django_spellbook.management.commands.spellbook_md_p.reporter import MarkdownReporter

# Define relative paths
INPUT_MD_DIR = 'markdown_testers'
INPUT_MD_FILENAME = 'test_djangolike_blog.md'
OUTPUT_HTML_DIR = 'html_testers'
OUTPUT_HTML_FILENAME = 'test_djangolike_blog_parser_output.html'
GOLDEN_HTML_DIR = 'html_goldens' # New directory for golden file
GOLDEN_HTML_FILENAME = 'test_djangolike_blog.html'

class TestDjangoLikeIntegrationWithParser(TestCase):
    """
    Performs an end-to-end integration test using the application's
    MarkdownParser class. Includes focused assertions targeting known issues
    and comparison against an OPTIMAL golden HTML file.
    """

    @classmethod
    def setUpClass(cls):
        """
        Reads markdown input file, processes via MarkdownParser, stores HTML string,
        and writes HTML output file once for the class. Loads OPTIMAL golden file.
        """
        super().setUpClass()

        cls.base_dir = os.path.dirname(os.path.abspath(__file__))
        cls.input_md_path = os.path.join(cls.base_dir, INPUT_MD_DIR, INPUT_MD_FILENAME)

        if not os.path.exists(cls.input_md_path):
            raise FileNotFoundError(f"MD input file not found: {cls.input_md_path}")

        try:
            with open(cls.input_md_path, 'r', encoding='utf-8') as f:
                markdown_text = f.read()
        except Exception as e:
            raise IOError(f"Error reading MD file {cls.input_md_path}: {e}")

        cls.reporter_buffer = io.StringIO()
        cls.reporter = MarkdownReporter(cls.reporter_buffer)
        try:
            parser = MarkdownParser(markdown_text=markdown_text, reporter=cls.reporter)
            cls.html_output = parser.get_html()
        except Exception as e:
             raise RuntimeError(f"MarkdownParser failed during setUpClass: {e}")

        cls.output_html_dir_path = os.path.join(cls.base_dir, OUTPUT_HTML_DIR)
        cls.output_html_file_path = os.path.join(cls.output_html_dir_path, OUTPUT_HTML_FILENAME)
        try:
            os.makedirs(cls.output_html_dir_path, exist_ok=True)
            with open(cls.output_html_file_path, 'w', encoding='utf-8') as f:
                f.write(cls.html_output)
        except Exception as e:
            print(f"\n[Test Warning] Could not write actual HTML output file {cls.output_html_file_path}: {e}")

        cls.golden_html_path = os.path.join(cls.base_dir, GOLDEN_HTML_DIR, GOLDEN_HTML_FILENAME)
        if not os.path.exists(cls.golden_html_path):
            cls.expected_html = None
            print(f"\n[Test Warning] OPTIMAL golden file not found: {cls.golden_html_path}. Golden test will fail.")
        else:
            try:
                with open(cls.golden_html_path, 'r', encoding='utf-8') as f:
                    cls.expected_html = f.read()
            except Exception as e:
                cls.expected_html = None
                print(f"\n[Test Warning] Error reading OPTIMAL golden file {cls.golden_html_path}: {e}. Golden test will fail.")

    # --- Golden File Test (Ultimate Goal) ---
    @unittest.expectedFailure
    def test_against_optimal_golden_file(self):
        """Compares the generated HTML against the OPTIMAL golden file."""
        self.assertIsNotNone(self.expected_html, f"OPTIMAL Golden file could not be loaded: {self.golden_html_path}")
        # This test WILL FAIL until all processor bugs and MD rendering issues are fixed.
        self.assertMultiLineEqual(
            self.html_output.strip(),
            self.expected_html.strip(),
            "Generated HTML does not match OPTIMAL golden file. Fix processor bugs!"
            )

    # --- Assertions Targeting Specific Remaining Issues ---

    @unittest.expectedFailure
    def test_markdown_lists_inside_custom_tag(self):
        """
        BUG CHECK: Asserts Markdown lists render correctly inside custom tags.
        (Currently fails - list renders as literal text with <br /> inside <p>).
        """
        expected_list_html = '<ul>\n<li>As should lists.</li>\n<li>Item two.</li>\n</ul>'
        section_regex = r'<section\s+.*?class="hero".*?id="hero-section".*?>(.*?)<\/section>'
        match = re.search(section_regex, self.html_output, re.DOTALL)
        self.assertTrue(match, "Could not find the hero section tag.")
        section_content = match.group(1) if match else ""
        self.assertIn(expected_list_html, section_content, "Markdown list did not render correctly inside section tag.")
        # Also assert the incorrect rendering is NOT present
        self.assertNotIn("- As should lists.<br />", section_content, "List appears to be rendering literally inside section.")

    def test_no_paragraph_splitting_around_inline_django_tags(self):
        """
        BUG CHECK: Asserts paragraphs are NOT split incorrectly around inline django tags.
        (Currently fails - paragraphs split and spurious `<code>`/`<p>` tags appear).
        """
        # Check for the known *incorrect* pattern (split paragraph + potential code tag)
        incorrect_pattern_static = "<p>`: </p>\n<django-tag>{% static 'css/blog_styles.css' %}</django-tag>" # Example check
        incorrect_pattern_url = "<p>`: </p>\n<django-tag>{% url 'post_detail' slug='test-drive-post' %}</django-tag>" # Example check

        self.assertFalse(incorrect_pattern_static in self.html_output, "Paragraph incorrectly split around static tag.")
        self.assertFalse(incorrect_pattern_url in self.html_output, "Paragraph incorrectly split around url tag.")

        # Check that the OPTIMAL structure (single paragraph) is present
        # This will fail until fixed.
        optimal_inline_block = "<p><code>Let's link to a static asset</code>: <django-tag>{% static 'css/blog_styles.css' %}</django-tag><br />\n<code>And here's a dynamic URL:</code> <django-tag>{% url 'post_detail' slug='test-drive-post' %}</django-tag><br />\nWe can even include another template snippet: <django-tag>{% include 'includes/sidebar.html' %}</django-tag></p>"
        self.assertIn(optimal_inline_block, self.html_output, "Inline django tags and text not rendering correctly in single paragraph.")

    def test_django_else_tag_preservation(self):
        """
        Asserts '{% else %}' is preserved via <django-tag>, not converted to <else>.
        (Currently fails - renders as <else>...</else>).
        """
        # Assert that the incorrect <else> tag is NOT present
        self.assertNotIn('<else>', self.html_output, "Found unexpected <else> tag.")
        self.assertNotIn('</else>', self.html_output, "Found unexpected </else> tag.")
        # Assert that the correct <django-tag> wrapper IS present
        self.assertIn('<django-tag>{% else %}</django-tag>', self.html_output, "Expected <django-tag>{% else %}</django-tag> not found.")

    def test_elif_tag_preservation(self):
        """
        Asserts '{% elif %}' is preserved via <django-tag>, not converted to <elif>.
        (Currently fails - renders as <elif>...</elif>).
        """
        # Assert that the incorrect <elif> tag is NOT present
        self.assertNotIn('<elif>', self.html_output, "Found unexpected <elif> tag.")
        self.assertNotIn('</elif>', self.html_output, "Found unexpected </elif> tag.")
        # Assert that the correct <django-tag> wrapper IS present
        self.assertIn('<django-tag>{% elif ', self.html_output, "Expected <django-tag>{% elif %}</django-tag> not found.")

    def test_custom_end_tags_are_consumed(self):
        """
        Asserts that custom end tags are REMOVED from the final output.
        (This *should* pass now for section/article based on the latest HTML).
        We removed the complex div case where it failed before. Add specific failing cases if needed.
        """
        # These should now pass for the current markdown content
        self.assertFalse('{% enddiv %}' in self.html_output, "Found literal '{% enddiv %}' in output.")
        self.assertFalse('{% endsection %}' in self.html_output, "Found literal '{% endsection %}' in output.")
        self.assertFalse('{% endaside %}' in self.html_output, "Found literal '{% endaside %}' in output.")
        self.assertFalse('{% endarticle %}' in self.html_output, "Found literal '{% endarticle %}' in output.")

    @unittest.expectedFailure
    def test_heading_inside_custom_tag(self):
        """
        BUG CHECK: Asserts H2 renders correctly inside the article tag.
        (Currently fails - renders as literal '##' inside <p>).
        """
        article_regex = r'<article\s+.*?id="post-body".*?>(.*?)<\/article>'
        match = re.search(article_regex, self.html_output, re.DOTALL)
        self.assertTrue(match, "Could not find the article tag.")
        article_content = match.group(1) if match else ""
        # This assertion will fail
        self.assertIn('<h2>Sub-heading Inside Article</h2>', article_content, "H2 did not render correctly inside article.")
        # Check for the incorrect rendering
        self.assertIn('<p>## Sub-heading Inside Article<br />', article_content, "H2 seems to be rendering literally inside <p> tag.")


    def test_aside_content_is_paragraph(self):
        """
        ISSUE CHECK: Asserts that indented content inside <aside> renders as <p>, not <pre><code>.
        (Currently fails due to source indentation + standard MD behavior).
        """
        aside_regex = r'<aside\s+.*?class="pull-quote".*?>(.*?)<\/aside>'
        match = re.search(aside_regex, self.html_output, re.DOTALL)
        self.assertTrue(match, "Could not find the aside tag.")
        aside_content = match.group(1) if match else ""
        # This assertion will fail
        self.assertIn('<p>This is an aside (pull quote)', aside_content, "Aside content should be a paragraph.")
        self.assertNotIn('<pre><code>', aside_content, "Aside content rendered unexpectedly as pre/code.")


# Allows running the tests directly from the command line
if __name__ == '__main__':
    unittest.main()
